__author__ = "aiXplain"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Duraikrishna Selvaraju, Thiago Castro Ferreira, Shreyas Sharma and Lucas Pavanelli
Date: September 1st 2022
Description:
    Pipeline Factory Class
"""
import json
import os
import logging
from typing import Dict, List, Optional, Text, Union
from aixplain.factories.pipeline_factory.utils import build_from_response
from aixplain.enums.data_type import DataType
from aixplain.enums.function import Function
from aixplain.enums.supplier import Supplier
from aixplain.modules.model import Model
from aixplain.modules.pipeline import Pipeline
from aixplain.utils import config
from aixplain.utils.file_utils import _request_with_retry
from urllib.parse import urljoin
from warnings import warn


class PipelineFactory:
    """A static class for creating and exploring Pipeline Objects.

    Attributes:
        backend_url (str): The URL for the backend.
    """

    aixplain_key = config.AIXPLAIN_API_KEY
    backend_url = config.BACKEND_URL

    @classmethod
    def get(cls, pipeline_id: Text, api_key: Optional[Text] = None) -> Pipeline:
        """Create a 'Pipeline' object from pipeline id

        Args:
            pipeline_id (Text): Pipeline ID of required pipeline.
            api_key (Optional[Text], optional): Pipeline API key. Defaults to None.

        Returns:
            Pipeline: Created 'Pipeline' object
        """
        resp = None
        try:
            url = urljoin(cls.backend_url, f"sdk/pipelines/{pipeline_id}")
            if api_key is not None:
                headers = {
                    "Authorization": f"Token {api_key}",
                    "Content-Type": "application/json",
                }
            elif cls.aixplain_key != "":
                headers = {
                    "x-aixplain-key": f"{cls.aixplain_key}",
                    "Content-Type": "application/json",
                }
            else:
                headers = {
                    "Authorization": f"Token {config.TEAM_API_KEY}",
                    "Content-Type": "application/json",
                }
            logging.info(f"Start service for GET Pipeline  - {url} - {headers}")
            r = _request_with_retry("get", url, headers=headers)
            resp = r.json()
            # set api key
            resp["api_key"] = config.TEAM_API_KEY
            if api_key is not None:
                resp["api_key"] = api_key
            pipeline = build_from_response(resp, load_architecture=True)
            return pipeline
        except Exception as e:
            logging.exception(e)
            status_code = 400
            if resp is not None and "statusCode" in resp:
                status_code = resp["statusCode"]
                message = resp["message"]
                message = f"Pipeline Creation: Status {status_code} - {message}"
            else:
                message = f"Pipeline Creation: Unspecified Error {e}"
            logging.error(message)
            raise Exception(f"Status {status_code}: {message}")

    @classmethod
    def create_asset_from_id(cls, pipeline_id: Text) -> Pipeline:
        warn(
            'This method will be deprecated in the next versions of the SDK. Use "get" instead.',
            DeprecationWarning,
            stacklevel=2,
        )
        return cls.get(pipeline_id)

    @classmethod
    def get_assets_from_page(cls, page_number: int) -> List[Pipeline]:
        """Get the list of pipelines from a given page

        Args:
            page_number (int): Page from which pipelines are to be listed
        Returns:
            List[Pipeline]: List of pipelines based on given filters
        """
        try:
            url = urljoin(cls.backend_url, f"sdk/pipelines/?pageNumber={page_number}")
            if cls.aixplain_key != "":
                headers = {
                    "x-aixplain-key": f"{cls.aixplain_key}",
                    "Content-Type": "application/json",
                }
            else:
                headers = {
                    "Authorization": f"Token {config.TEAM_API_KEY}",
                    "Content-Type": "application/json",
                }
            r = _request_with_retry("get", url, headers=headers)
            resp = r.json()
            logging.info(f"Listing Pipelines: Status of getting Pipelines on Page {page_number}: {resp}")
            all_pipelines = resp["items"]
            pipeline_list = [build_from_response(pipeline_info_json) for pipeline_info_json in all_pipelines]
            return pipeline_list
        except Exception as e:
            error_message = f"Listing Pipelines: Error in getting Pipelines on Page {page_number}: {e}"
            logging.error(error_message, exc_info=True)
            return []

    @classmethod
    def get_first_k_assets(cls, k: int) -> List[Pipeline]:
        """Gets the first k given pipelines based on the provided task and language filters

        Args:
            k (int): Number of pipelines to get
        Returns:
            List[Pipeline]: List of pipelines based on given filters
        """
        try:
            pipeline_list = []
            assert k > 0
            for page_number in range(k // 10 + 1):
                pipeline_list += cls.get_assets_from_page(page_number)
            return pipeline_list
        except Exception as e:
            error_message = f"Listing Pipelines: Error in getting {k} Pipelines: {e}"
            logging.error(error_message, exc_info=True)
            return []

    @classmethod
    def list(
        cls,
        query: Optional[Text] = None,
        functions: Optional[Union[Function, List[Function]]] = None,
        suppliers: Optional[Union[Supplier, List[Supplier]]] = None,
        models: Optional[Union[Model, List[Model]]] = None,
        input_data_types: Optional[Union[DataType, List[DataType]]] = None,
        output_data_types: Optional[Union[DataType, List[DataType]]] = None,
        page_number: int = 0,
        page_size: int = 20,
        drafts_only: bool = False,
    ) -> Dict:

        url = urljoin(cls.backend_url, "sdk/pipelines/paginate")
        if cls.aixplain_key != "":
            headers = {
                "x-aixplain-key": f"{cls.aixplain_key}",
                "Content-Type": "application/json",
            }
        else:
            headers = {
                "Authorization": f"Token {config.TEAM_API_KEY}",
                "Content-Type": "application/json",
            }

        assert 0 < page_size <= 100, "Pipeline List Error: Page size must be greater than 0 and not exceed 100."
        payload = {
            "pageSize": page_size,
            "pageNumber": page_number,
            # "sort": [{"field": "createdAt", "dir": -1}],
            "draftsOnly": drafts_only,
        }

        if query is not None:
            payload["q"] = str(query)

        if functions is not None:
            if isinstance(functions, Function) is True:
                functions = [functions]
            payload["functions"] = [function.value for function in functions]

        if suppliers is not None:
            if isinstance(suppliers, Supplier) is True:
                suppliers = [suppliers]
            payload["suppliers"] = [supplier.value for supplier in suppliers]

        if models is not None:
            if isinstance(models, Model) is True:
                models = [models]
            payload["models"] = [model.id for model in models]

        if input_data_types is not None:
            if isinstance(input_data_types, DataType) is True:
                input_data_types = [input_data_types]
            payload["inputDataTypes"] = [data_type.value for data_type in input_data_types]

        if output_data_types is not None:
            if isinstance(output_data_types, DataType) is True:
                output_data_types = [output_data_types]
            payload["inputDataTypes"] = [data_type.value for data_type in output_data_types]

        logging.info(f"Start service for POST List Pipeline - {url} - {headers} - {json.dumps(payload)}")
        r = _request_with_retry("post", url, headers=headers, json=payload)
        resp = r.json()

        pipelines, page_total, total = [], 0, 0
        if "items" in resp:
            results = resp["items"]
            page_total = resp["pageTotal"]
            total = resp["total"]
            logging.info(f"Response for POST List Pipeline - Page Total: {page_total} / Total: {total}")
            for pipeline in results:
                pipelines.append(build_from_response(pipeline))
        return {
            "results": pipelines,
            "page_total": page_total,
            "page_number": page_number,
            "total": total,
        }

    @classmethod
    def init(cls, name: Text, api_key: Optional[Text] = None) -> Pipeline:
        """Initialize a new Pipeline

        Args:
            name (Text): Pipeline Name
            api_key (Optional[Text], optional): Team API Key to create the Pipeline. Defaults to None.

        Returns:
            Pipeline: instance of the new pipeline
        """
        if api_key is None:
            api_key = config.TEAM_API_KEY
        return Pipeline(
            id="",
            name=name,
            api_key=api_key,
            nodes=[],
            links=[],
            instance=None,
        )

    @classmethod
    def create(
        cls,
        name: Text,
        pipeline: Union[Text, Dict],
        api_key: Optional[Text] = None,
    ) -> Pipeline:
        """Draft Pipeline Creation

        Args:
            name (Text): Pipeline Name
            pipeline (Union[Text, Dict]): Pipeline as a Python dictionary or in a JSON file
            api_key (Optional[Text], optional): Team API Key to create the Pipeline. Defaults to None.

        Raises:
            Exception: Currently just the creation of draft pipelines are supported

        Returns:
            Pipeline: instance of the new pipeline
        """
        try:
            if isinstance(pipeline, str) is True:
                _, ext = os.path.splitext(pipeline)
                assert (
                    os.path.exists(pipeline) and ext == ".json"
                ), "Pipeline Creation Error: Make sure the pipeline to be saved is in a JSON file."
                with open(pipeline) as f:
                    pipeline = json.load(f)

            for i, node in enumerate(pipeline["nodes"]):
                if "functionType" in node and node["functionType"] == "AI":
                    pipeline["nodes"][i]["functionType"] = pipeline["nodes"][i]["functionType"].lower()
            # prepare payload
            payload = {
                "name": name,
                "status": "draft",
                "architecture": pipeline,
            }
            url = urljoin(cls.backend_url, "sdk/pipelines")
            api_key = api_key if api_key is not None else config.TEAM_API_KEY
            headers = {
                "Authorization": f"Token {api_key}",
                "Content-Type": "application/json",
            }
            logging.info(f"Start service for POST Create Pipeline - {url} - {headers} - {json.dumps(payload)}")
            r = _request_with_retry("post", url, headers=headers, json=payload)
            response = r.json()

            return Pipeline(response["id"], name, api_key)
        except Exception as e:
            raise Exception(e)



__author__ = "shreyassharma"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Duraikrishna Selvaraju, Thiago Castro Ferreira, Shreyas Sharma and Lucas Pavanelli
Date: February 1st 2023
Description:
    Corpus Class
"""
import logging

from aixplain.enums.function import Function
from aixplain.enums.license import License
from aixplain.enums.onboard_status import OnboardStatus
from aixplain.enums.privacy import Privacy
from aixplain.modules.asset import Asset
from aixplain.modules.data import Data
from aixplain.utils import config
from aixplain.utils.file_utils import _request_with_retry
from typing import Any, List, Optional, Text
from urllib.parse import urljoin


class Corpus(Asset):
    def __init__(
        self,
        id: Text,
        name: Text,
        description: Text,
        data: List[Data],
        onboard_status: OnboardStatus,
        functions: List[Function] = [],
        tags: List[Text] = [],
        license: Optional[License] = None,
        privacy: Privacy = Privacy.PRIVATE,
        supplier: Text = "aiXplain",
        version: Text = "1.0",
        length: Optional[int] = None,
        **kwargs,
    ) -> None:
        """Corpus Class.

        Description:
            Corpus is general-purpose collection of data that can be processed and used to create task-specific datasets.

        Args:
            id (Text): Corpus ID
            name (Text): Corpus Name
            description (Text): description of the corpus
            data (List[Data]): List of data which the corpus consists of
            onboard_status (OnboardStatus): onboard status
            functions (List[Function], optional): AI functions in which the corpus is suggested to be used to. Defaults to [].
            tags (List[Text], optional): description tags. Defaults to [].
            license (Optional[License], optional): Corpus license. Defaults to None.
            privacy (Privacy, optional): Corpus privacy info. Defaults to Privacy.PRIVATE.
            supplier (Text, optional): Corpus supplier. Defaults to "aiXplain".
            version (Text, optional): Corpus version. Defaults to "1.0".
            length (Optional[int], optional): Number of rows in the Corpus. Defaults to None.
        """
        super().__init__(
            id=id, name=name, description=description, supplier=supplier, version=version, license=license, privacy=privacy
        )
        if isinstance(onboard_status, str):
            onboard_status = OnboardStatus(onboard_status)
        self.onboard_status = onboard_status
        self.functions = functions
        self.tags = tags
        self.data = data
        self.length = length
        self.kwargs = kwargs

    def __repr__(self):
        return f"<Corpus: {self.name}>"

    def delete(self) -> None:
        """Delete Corpus service"""
        try:
            url = urljoin(config.BACKEND_URL, f"sdk/corpora/{self.id}")
            headers = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
            logging.info(f"Start service for DELETE Corpus  - {url} - {headers}")
            r = _request_with_retry("delete", url, headers=headers)
            if r.status_code != 200:
                raise Exception()
        except Exception:
            message = "Corpus Deletion Error: Make sure the corpus exists and you are the owner."
            logging.error(message)
            raise Exception(f"{message}")



import pytest
import unittest.mock as mock


from aixplain.enums import DataType
from aixplain.modules.pipeline.designer.base import (
    Node,
    Link,
    Param,
    ParamProxy,
    Inputs,
    Outputs,
    InputParam,
    OutputParam,
)

from aixplain.modules.pipeline.designer.enums import (
    ParamType,
    NodeType,
)

from aixplain.modules.pipeline.designer.mixins import LinkableMixin
from aixplain.modules.pipeline.designer.pipeline import DesignerPipeline


def test_create_node():

    pipeline = DesignerPipeline()

    class BareNode(Node):
        pass

    with mock.patch("aixplain.modules.pipeline.designer.Node.attach_to") as mock_attach_to:
        node = BareNode(number=3, label="FOO")
        mock_attach_to.assert_not_called()
        assert isinstance(node.inputs, Inputs)
        assert isinstance(node.outputs, Outputs)
        assert node.number == 3
        assert node.label == "FOO"

    class FooNodeInputs(Inputs):
        pass

    class FooNodeOutputs(Outputs):
        pass

    class FooNode(Node[FooNodeInputs, FooNodeOutputs]):
        inputs_class = FooNodeInputs
        outputs_class = FooNodeOutputs

    with mock.patch("aixplain.modules.pipeline.designer.Node.attach_to") as mock_attach_to:
        node = FooNode(pipeline=pipeline, number=3, label="FOO")
        mock_attach_to.assert_called_once_with(pipeline)
        assert isinstance(node.inputs, FooNodeInputs)
        assert isinstance(node.outputs, FooNodeOutputs)
        assert node.number == 3
        assert node.label == "FOO"


def test_node_attach_to():

    pipeline = DesignerPipeline()

    class BareNode(Node):
        pass

    node = BareNode()
    with pytest.raises(AssertionError) as excinfo:
        node.attach_to(pipeline)

    assert "Node type not set" in str(excinfo.value)

    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    a = AssetNode()
    b = AssetNode()
    c = AssetNode()
    d = AssetNode(number=8)
    e = AssetNode(number=8)

    a.attach_to(pipeline)
    b.attach_to(pipeline)
    assert b.pipeline is pipeline
    assert b.number == 1
    assert b.label == "ASSET(ID=1)"
    assert b in pipeline.nodes
    assert len(pipeline.nodes) == 2

    c.attach_to(pipeline)
    assert c.pipeline is pipeline
    assert c.number == 2
    assert c.label == "ASSET(ID=2)"
    assert c in pipeline.nodes
    assert len(pipeline.nodes) == 3

    d.attach_to(pipeline)
    assert d.pipeline is pipeline
    assert d.number == 8
    assert d.label == "ASSET(ID=8)"
    assert d in pipeline.nodes
    assert len(pipeline.nodes) == 4

    with pytest.raises(AssertionError) as excinfo:
        e.attach_to(pipeline)

    assert "Node number already exists" in str(excinfo.value)


def test_node_serialize():
    pipeline = DesignerPipeline()

    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    node = AssetNode()

    with mock.patch.object(node.inputs, "serialize") as mock_inputs_serialize:
        with mock.patch.object(node.outputs, "serialize") as mock_outputs_serialize:
            assert node.serialize() == {
                "number": node.number,
                "type": NodeType.ASSET,
                "inputValues": mock_inputs_serialize.return_value,
                "outputValues": mock_outputs_serialize.return_value,
                "label": node.label,
            }
            node.attach_to(pipeline)
            mock_inputs_serialize.assert_called_once()
            mock_outputs_serialize.assert_called_once()
            mock_inputs_serialize.reset_mock()
            mock_outputs_serialize.reset_mock()

            assert node.serialize() == {
                "number": node.number,
                "type": NodeType.ASSET,
                "inputValues": mock_inputs_serialize.return_value,
                "outputValues": mock_outputs_serialize.return_value,
                "label": node.label,
            }
            mock_inputs_serialize.assert_called_once()
            mock_outputs_serialize.assert_called_once()


def test_create_param():
    class TypedParam(Param):
        param_type = ParamType.INPUT

    with mock.patch("aixplain.modules.pipeline.designer.Param.attach_to") as mock_attach_to:
        param = TypedParam(
            code="param",
            data_type=DataType.TEXT,
            value="foo",
        )
        mock_attach_to.assert_not_called()

    assert param.code == "param"
    assert param.data_type == DataType.TEXT
    assert param.value == "foo"
    assert param.param_type == ParamType.INPUT

    with mock.patch("aixplain.modules.pipeline.designer.Param.attach_to") as mock_attach_to:
        param = TypedParam(
            code="param",
            data_type=DataType.TEXT,
            value="foo",
            param_type=ParamType.OUTPUT,
        )
        mock_attach_to.assert_not_called()

    assert param.code == "param"
    assert param.data_type == DataType.TEXT
    assert param.value == "foo"
    assert param.param_type == ParamType.INPUT

    class UnTypedParam(Param):
        pass

    with mock.patch("aixplain.modules.pipeline.designer.Param.attach_to") as mock_attach_to:
        param = UnTypedParam(
            code="param",
            data_type=DataType.TEXT,
            value="foo",
            param_type=ParamType.OUTPUT,
        )
        mock_attach_to.assert_not_called()

    assert param.param_type == ParamType.OUTPUT

    with mock.patch("aixplain.modules.pipeline.designer.Param.attach_to") as mock_attach_to:
        param = UnTypedParam(
            code="param",
            data_type=DataType.TEXT,
            value="foo",
            param_type=ParamType.INPUT,
        )
        mock_attach_to.assert_not_called()

    assert param.param_type == ParamType.INPUT

    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    node = AssetNode()

    with mock.patch("aixplain.modules.pipeline.designer.Param.attach_to") as mock_attach_to:
        param = UnTypedParam(
            code="param",
            data_type=DataType.TEXT,
            value="foo",
            param_type=ParamType.INPUT,
            node=node,
        )
        mock_attach_to.assert_called_once_with(node)


@pytest.mark.parametrize(
    "param_cls, expected_param_type",
    [
        (InputParam, ParamType.INPUT),
        (OutputParam, ParamType.OUTPUT),
    ],
)
def test_create_input_output_param(param_cls, expected_param_type):
    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    node = AssetNode()

    with mock.patch("aixplain.modules.pipeline.designer.Param.attach_to") as mock_attach_to:
        param = param_cls(code="param", data_type=DataType.TEXT, value="foo", node=node)
        mock_attach_to.assert_called_once_with(node)
        assert param.code == "param"
        assert param.data_type == DataType.TEXT
        assert param.value == "foo"
        assert param.param_type == expected_param_type
        assert not param.node


def test_param_attach_to():
    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    node = AssetNode()

    class NoTypeParam(Param):
        pass

    param = NoTypeParam(code="param", data_type=DataType.TEXT, value="foo")
    with pytest.raises(AssertionError) as excinfo:
        param.attach_to(node)

    assert "Param type not set" in str(excinfo.value)

    input = InputParam(code="input", data_type=DataType.TEXT, value="foo")

    with mock.patch.object(node.inputs, "add_param") as mock_add_param:
        input.attach_to(node)
        mock_add_param.assert_called_once_with(input)
    assert input.node is node

    with pytest.raises(AssertionError) as excinfo:
        input.attach_to(node)

    assert "Param already attached to a node" in str(excinfo.value)

    output = OutputParam(code="output", data_type=DataType.TEXT, value="bar")

    with mock.patch.object(node.outputs, "add_param") as mock_add_param:
        output.attach_to(node)
        mock_add_param.assert_called_once_with(output)
    assert output.node is node


def test_param_link():
    input = InputParam(code="input", data_type=DataType.TEXT, value="foo")
    output = OutputParam(code="output", data_type=DataType.TEXT, value="bar")

    with pytest.raises(AssertionError) as excinfo:
        output.link(input)

    assert "Param not attached to a node" in str(excinfo.value)

    class AssetNode(Node, LinkableMixin):
        type: NodeType = NodeType.ASSET

    a = AssetNode()
    b = AssetNode()

    output = OutputParam(code="output", data_type=DataType.TEXT, value="bar")
    output.node = a
    input = InputParam(code="input", data_type=DataType.TEXT, value="foo")
    input.node = b

    with pytest.raises(AssertionError) as excinfo:
        input.link(output)

    assert "Invalid param type" in str(excinfo.value)

    with pytest.raises(AssertionError) as excinfo:
        output.link(input)

    assert "Param not registered as output" in str(excinfo.value)

    output = OutputParam(code="output", data_type=DataType.TEXT, value="bar", node=a)
    input = InputParam(code="input", data_type=DataType.TEXT, value="foo", node=b)

    with mock.patch.object(input, "back_link") as mock_back_link:
        output.link(input)
        mock_back_link.assert_called_once_with(output)


def test_param_back_link():
    input = InputParam(code="input", data_type=DataType.TEXT, value="foo")
    output = OutputParam(code="output", data_type=DataType.TEXT, value="bar")

    with pytest.raises(AssertionError) as excinfo:
        input.back_link(output)

    assert "Param not attached to a node" in str(excinfo.value)

    class AssetNode(Node, LinkableMixin):
        type: NodeType = NodeType.ASSET

    a = AssetNode()
    b = AssetNode()

    output = OutputParam(code="output", data_type=DataType.TEXT, value="bar")
    output.node = a
    input = InputParam(code="input", data_type=DataType.TEXT, value="foo")
    input.node = b

    with pytest.raises(AssertionError) as excinfo:
        output.back_link(input)

    assert "Invalid param type" in str(excinfo.value)

    with pytest.raises(AssertionError) as excinfo:
        input.back_link(output)

    assert "Param not registered as input" in str(excinfo.value)

    output = OutputParam(code="output", data_type=DataType.TEXT, value="bar", node=a)
    input = InputParam(code="input", data_type=DataType.TEXT, value="foo", node=b)

    with mock.patch.object(a, "link") as mock_link:
        input.back_link(output)
        mock_link.assert_called_once_with(b, output, input)


def test_create_pipeline():
    pipeline = DesignerPipeline()

    assert pipeline.nodes == []
    assert pipeline.links == []
    assert not pipeline.instance


def test_link_create():
    class AssetNode(Node, LinkableMixin):
        type: NodeType = NodeType.ASSET

    a = AssetNode()
    b = AssetNode()

    with pytest.raises(AssertionError) as excinfo:
        link = Link(
            from_node=a,
            to_node=b,
            from_param="output",
            to_param="input",
        )

    assert "Invalid from param" in str(excinfo.value)

    a.outputs.create_param("output", DataType.TEXT, "foo")

    with pytest.raises(AssertionError) as excinfo:
        link = Link(
            from_node=a,
            to_node=b,
            from_param="output",
            to_param="input",
        )

    assert "Invalid to param" in str(excinfo.value)

    b.inputs.create_param("input", DataType.TEXT, "bar")

    link = Link(
        from_node=a,
        to_node=b,
        from_param="output",
        to_param="input",
    )

    assert link.from_node == a
    assert link.to_node == b
    assert link.from_param == "output"
    assert link.to_param == "input"

    pipeline = DesignerPipeline()

    with mock.patch("aixplain.modules.pipeline.designer.Link.attach_to") as mock_attach_to:
        link = Link(
            from_node=a,
            to_node=b,
            from_param="output",
            to_param="input",
            pipeline=pipeline,
        )
        mock_attach_to.assert_called_once_with(pipeline)


def test_link_attach_to():

    pipeline = DesignerPipeline()

    class AssetNode(Node, LinkableMixin):
        type: NodeType = NodeType.ASSET

    a = AssetNode()
    b = AssetNode()

    a.outputs.create_param("output", DataType.TEXT, "foo")
    b.inputs.create_param("input", DataType.TEXT, "bar")

    link = Link(
        from_node=a,
        to_node=b,
        from_param="output",
        to_param="input",
    )

    with mock.patch.object(a, "attach_to") as mock_a_attach_to:
        with mock.patch.object(b, "attach_to") as mock_b_attach_to:
            link.attach_to(pipeline)
            mock_a_attach_to.assert_called_once_with(pipeline)
            mock_b_attach_to.assert_called_once_with(pipeline)
            assert link.pipeline is pipeline
            assert link in pipeline.links

    a = AssetNode(pipeline=pipeline)
    b = AssetNode(pipeline=pipeline)
    a.outputs.create_param("output", DataType.TEXT, "foo")
    b.inputs.create_param("input", DataType.TEXT, "bar")

    link = Link(
        from_node=a,
        to_node=b,
        from_param="output",
        to_param="input",
    )

    with mock.patch.object(a, "attach_to") as mock_a_attach_to:
        with mock.patch.object(b, "attach_to") as mock_b_attach_to:
            link.attach_to(pipeline)
            mock_a_attach_to.assert_not_called()
            mock_b_attach_to.assert_not_called()
            assert link.pipeline is pipeline
            assert link in pipeline.links

    with pytest.raises(AssertionError) as excinfo:
        link.attach_to(pipeline)

    assert "Link already attached to a pipeline" in str(excinfo.value)


def test_link_serialize():
    pipeline = DesignerPipeline()

    class AssetNode(Node, LinkableMixin):
        type: NodeType = NodeType.ASSET

    a = AssetNode()
    b = AssetNode()
    a.outputs.create_param("output", DataType.TEXT, "foo")
    b.inputs.create_param("input", DataType.TEXT, "bar")

    link = Link(
        from_node=a,
        to_node=b,
        from_param="output",
        to_param="input",
    )

    with pytest.raises(AssertionError) as excinfo:
        link.serialize()

    assert "From node number not set" in str(excinfo.value)
    a.attach_to(pipeline)

    with pytest.raises(AssertionError) as excinfo:
        link.serialize()

    assert "To node number not set" in str(excinfo.value)
    b.attach_to(pipeline)

    link = Link(
        from_node=a,
        to_node=b,
        from_param="output",
        to_param="input",
    )

    assert link.serialize() == {
        "from": a.number,
        "to": b.number,
        "paramMapping": [
            {"from": "output", "to": "input"},
        ],
    }


def test_create_param_proxy():
    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    node = AssetNode()

    param_proxy = ParamProxy(node)
    assert param_proxy.node is node
    assert param_proxy._params == []


def test_param_proxy_add_param():
    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    node = AssetNode()

    param_proxy = ParamProxy(node)

    class FooParam(Param):
        pass

    param = FooParam(code="foo", data_type=DataType.TEXT)
    param_proxy.add_param(param)
    assert param in param_proxy._params
    assert hasattr(param_proxy, "foo")
    assert param_proxy.foo is param
    assert param_proxy.foo.code == "foo"
    assert param_proxy.foo.data_type == DataType.TEXT

    with pytest.raises(ValueError) as excinfo:
        param_proxy.add_param(param)

    assert "Parameter with code 'foo' already exists." in str(excinfo.value)


def test_param_proxy_create_param():
    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    node = AssetNode()

    param_proxy = ParamProxy(node)

    with mock.patch.object(param_proxy, "_create_param") as mock_create_param:
        with mock.patch.object(param_proxy, "add_param") as mock_add_param:
            param = param_proxy.create_param("foo", DataType.TEXT, "bar", is_required=True)
            mock_create_param.assert_called_once_with("foo", DataType.TEXT, "bar")
            mock_add_param.assert_called_once_with(param)
            assert param.is_required is True


def test_param_proxy_attr_access():
    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    node = AssetNode()

    param_proxy = ParamProxy(node)

    class FooParam(Param):
        pass

    param = FooParam(code="foo", data_type=DataType.TEXT)
    param_proxy.add_param(param)

    assert param in param_proxy
    assert "foo" in param_proxy
    assert param_proxy["foo"] is param
    assert param_proxy.foo is param

    with pytest.raises(KeyError) as excinfo:
        param_proxy["bar"]

    assert "'bar'" in str(excinfo.value)


def test_node_link():
    class AssetNode(Node, LinkableMixin):
        type: NodeType = NodeType.ASSET

    a = AssetNode()
    b = AssetNode()

    output = OutputParam(code="output", data_type=DataType.TEXT, value="bar", node=a)
    input = InputParam(code="input", data_type=DataType.TEXT, value="foo", node=b)

    # here too lazy to mock Link class properly
    # checking the output instance instead
    link = a.link(b, from_param=output, to_param=input)
    assert isinstance(link, Link)
    assert link.from_node == a
    assert link.to_node == b
    assert link.from_param == "output"
    assert link.to_param == "input"


def test_pipeline_add_node():
    pipeline = DesignerPipeline()

    class InputNode(Node):
        type: NodeType = NodeType.INPUT

    node = InputNode()
    pipeline.add_node(node)
    assert pipeline.nodes == [node]
    assert pipeline.links == []

    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    node1 = AssetNode()
    with mock.patch.object(node1, "attach_to") as mock_attach_to:
        pipeline.add_node(node1)
        mock_attach_to.assert_called_once_with(pipeline)


def test_pipeline_add_nodes():
    pipeline = DesignerPipeline()

    class InputNode(Node):
        type: NodeType = NodeType.INPUT

    node = InputNode()

    with mock.patch.object(pipeline, "add_node") as mock_add_node:
        pipeline.add_nodes(node)
        assert mock_add_node.call_count == 1

    node1 = InputNode()
    node2 = InputNode()

    with mock.patch.object(pipeline, "add_node") as mock_add_node:
        pipeline.add_nodes(node1, node2)
        assert mock_add_node.call_count == 2


def test_pipeline_add_link():
    pipeline = DesignerPipeline()

    class AssetNode(Node):
        type: NodeType = NodeType.ASSET

    a = AssetNode()
    a.outputs.create_param("output", DataType.TEXT)
    b = AssetNode()
    b.inputs.create_param("input", DataType.TEXT)

    link = Link(from_node=a, to_node=b, from_param="output", to_param="input")
    pipeline.add_link(link)

    with mock.patch.object(link, "attach_to") as mock_attach_to:
        pipeline.add_link(link)
        mock_attach_to.assert_called_once_with(pipeline)



__author__ = "lucaspavanelli"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import uuid
import json
from dotenv import load_dotenv

load_dotenv()
import requests
import requests_mock
from aixplain.utils import config
from aixplain.factories import ModelFactory
from aixplain.factories import FinetuneFactory
from aixplain.modules import Model, Finetune
from aixplain.modules.finetune import Hyperparameters
from aixplain.enums import Function
from urllib.parse import urljoin

import pytest

FIXED_HEADER = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
COST_ESTIMATION_URL = f"{config.BACKEND_URL}/sdk/finetune/cost-estimation"
COST_ESTIMATION_FILE = "tests/unit/mock_responses/cost_estimation_response.json"
FINETUNE_URL = f"{config.BACKEND_URL}/sdk/finetune"
FINETUNE_FILE = "tests/unit/mock_responses/finetune_response.json"
FINETUNE_STATUS_FILE = "tests/unit/mock_responses/finetune_status_response.json"
FINETUNE_STATUS_FILE_2 = "tests/unit/mock_responses/finetune_status_response_2.json"
PERCENTAGE_EXCEPTION_FILE = "tests/unit/data/create_finetune_percentage_exception.json"
MODEL_FILE = "tests/unit/mock_responses/model_response.json"
MODEL_URL = f"{config.BACKEND_URL}/sdk/models"
LIST_FINETUNABLE_FILE = "tests/unit/mock_responses/list_models_response.json"


def read_data(data_path):
    return json.load(open(data_path, "r"))


@pytest.fixture(scope="module", params=read_data(PERCENTAGE_EXCEPTION_FILE))
def percentage_exception_map(request):
    return request.param


def test_create():
    model_map = read_data(MODEL_FILE)
    with requests_mock.Mocker() as mock:
        test_model = "test_asset_id"
        url = f"{MODEL_URL}/{test_model}"
        mock.get(url, headers=FIXED_HEADER, json=model_map)
        cost_estimation_map = read_data(COST_ESTIMATION_FILE)
        mock.post(COST_ESTIMATION_URL, headers=FIXED_HEADER, json=cost_estimation_map)
        finetune = FinetuneFactory.create("", [], test_model, prompt_template="test", hyperparameters=Hyperparameters())
    assert finetune is not None
    assert finetune.model.id == test_model
    assert finetune.cost.to_dict() == cost_estimation_map

def test_create_exception():
    model_map = read_data(MODEL_FILE)
    with requests_mock.Mocker() as mock:
        test_model = "test_asset_id"
        url = f"{MODEL_URL}/{test_model}"
        mock.get(url, headers=FIXED_HEADER, json=model_map)
        mock.post(COST_ESTIMATION_URL, exc=requests.exceptions.ConnectTimeout)
        finetune = FinetuneFactory.create("", [], test_model, prompt_template="test", hyperparameters=Hyperparameters())
    assert finetune is None


def test_create_train_dev_percentage(percentage_exception_map):
    with requests_mock.Mocker() as mock:
        cost_estimation_map = read_data(COST_ESTIMATION_FILE)
        mock.post(COST_ESTIMATION_URL, headers=FIXED_HEADER, json=cost_estimation_map)
        test_model = Model("", "")
        with pytest.raises(Exception) as exception_info:
            FinetuneFactory.create(
                "",
                [],
                test_model,
                train_percentage=percentage_exception_map["train_percentage"],
                dev_percentage=percentage_exception_map["dev_percentage"],
            )
    assert exception_info.type is AssertionError


def test_start():
    finetune_map = read_data(FINETUNE_FILE)
    model_map = read_data(MODEL_FILE)
    asset_id = finetune_map["id"]
    with requests_mock.Mocker() as mock:
        url = f"{MODEL_URL}/{asset_id}"
        mock.get(url, headers=FIXED_HEADER, json=model_map)
        mock.post(FINETUNE_URL, headers=FIXED_HEADER, json=finetune_map)
        test_model = Model("", "")
        finetune = Finetune("", [], test_model, None)
        fine_tuned_model = finetune.start()
    assert fine_tuned_model is not None
    assert fine_tuned_model.id == model_map["id"]

@pytest.mark.parametrize(
    "input_path,after_epoch,training_loss,validation_loss", 
    [
        (FINETUNE_STATUS_FILE, None, 0.4, 0.0217), 
        (FINETUNE_STATUS_FILE, 1, 0.2, 0.0482),
        (FINETUNE_STATUS_FILE_2, None, 2.657801408034, 2.596168756485), 
        (FINETUNE_STATUS_FILE_2, 0, None, 2.684150457382)
    ]
)
def test_check_finetuner_status(input_path, after_epoch, training_loss, validation_loss):
    model_map = read_data(input_path)
    asset_id = "test_id"
    with requests_mock.Mocker() as mock:
        test_model = Model(asset_id, "")
        url = urljoin(config.BACKEND_URL, f"sdk/finetune/{asset_id}/ml-logs")
        mock.get(url, headers=FIXED_HEADER, json=model_map)
        status = test_model.check_finetune_status(after_epoch=after_epoch)
    assert status.status.value == model_map["finetuneStatus"]
    assert status.model_status.value == model_map["modelStatus"]
    assert status.training_loss == training_loss
    assert status.validation_loss == validation_loss


@pytest.mark.parametrize("is_finetunable", [True, False])
def test_list_finetunable_models(is_finetunable):
    list_map = read_data(LIST_FINETUNABLE_FILE)
    with requests_mock.Mocker() as mock:
        print(f"is_finetunable: {is_finetunable}")
        url = f"{config.BACKEND_URL}/sdk/models/paginate"
        mock.post(url, headers=FIXED_HEADER, json=list_map)
        result = ModelFactory.list(function=Function.TRANSLATION, is_finetunable=is_finetunable, page_number=0, page_size=5)
        print(result)
    assert result["page_total"] == 5
    assert result["page_number"] == 0
    model_list = result["results"]
    assert len(model_list) > 0
    for model_index in range(len(model_list)):
        assert model_list[model_index].id == list_map["items"][model_index]["id"]


import pytest

from aixplain.enums import DataType
from aixplain.factories import PipelineFactory
from aixplain.modules.pipeline.designer import (
    Link,
    Operation,
    Route,
    RouteType,
)
from aixplain.modules import Pipeline
from aixplain.modules.pipeline.designer import AssetNode
from uuid import uuid4


@pytest.fixture
def pipeline():
    # Setup: Initialize the pipeline
    pipeline = PipelineFactory.init(
        name=str(uuid4()),
    )

    # Yield control back to the test function
    yield pipeline

    # Teardown: Ensure the pipeline is deleted
    if pipeline is not None:
        pipeline.delete()


def test_create_asr_pipeline(pipeline):
    # add nodes to the pipeline
    input = pipeline.input()
    model1 = AssetNode(asset_id="60ddefab8d38c51c5885ee38")
    pipeline.add_node(model1)

    model2 = AssetNode(asset_id="60ddefd68d38c51c588608f1")
    pipeline.add_node(model2)

    # link the nodes
    link1 = Link(
        from_node=input,
        to_node=model1,
        from_param="input",
        to_param="source_audio",
    )
    pipeline.add_link(link1)

    link2 = Link(
        from_node=model1,
        to_node=model2,
        from_param="data",
        to_param="text",
    )
    pipeline.add_link(link2)

    # use the output of the last node
    model1.use_output("data")
    model2.use_output("data")

    # save the pipeline as draft
    pipeline.save()

    assert isinstance(pipeline, Pipeline)
    assert pipeline.id != ""


def test_create_mt_pipeline_and_run(pipeline):
    # add nodes to the pipeline
    input = pipeline.input()
    model1 = pipeline.translation(asset_id="60ddef828d38c51c5885d491")
    output = pipeline.output()

    # link the nodes
    input.link(
        to_node=model1,
        from_param=input.outputs.input,
        to_param=model1.inputs.text,
    )

    # use the output of the last node
    model1.link(
        to_node=output,
        from_param=model1.outputs.data,
        to_param=output.inputs.output,
    )

    # save the pipeline as an asset
    pipeline.save(save_as_asset=True)

    assert isinstance(pipeline, Pipeline)
    assert pipeline.id != ""

    pipeline = PipelineFactory.get(pipeline.id)

    # run the pipeline
    output = pipeline.run(
        "https://aixplain-platform-assets.s3.amazonaws.com/samples/en/CPAC1x2.txt",
        **{"batchmode": False, "version": "2.0"},
    )
    assert output["status"] == "SUCCESS"


def test_routing_pipeline(pipeline):

    TRANSLATION_ASSET = "60ddefae8d38c51c5885eff7"
    SPEECH_RECOGNITION_ASSET = "621cf3fa6442ef511d2830af"

    input = pipeline.input()
    translation = pipeline.asset(TRANSLATION_ASSET)
    speech_recognition = pipeline.asset(SPEECH_RECOGNITION_ASSET)

    input.route(
        translation.inputs.text, speech_recognition.inputs.source_audio
    )

    translation.use_output("data")
    speech_recognition.use_output("data")

    pipeline.save()

    output = pipeline.run("This is a sample text!")

    assert output["status"] == "SUCCESS"
    assert output.get("data") is not None
    assert len(output["data"]) > 0
    assert output["data"][0].get("segments") is not None
    assert len(output["data"][0]["segments"]) > 0


def test_scripting_pipeline(pipeline):

    SPEAKER_DIARIZATION_AUDIO_ASSET = "62fab6ecb39cca09ca5bc365"
    SPEECH_RECOGNITION_ASSET = "621cf3fa6442ef511d2830af"

    input = pipeline.input()

    segmentor = pipeline.speaker_diarization_audio(
        asset_id=SPEAKER_DIARIZATION_AUDIO_ASSET
    )

    speech_recognition = pipeline.speech_recognition(
        asset_id=SPEECH_RECOGNITION_ASSET
    )

    script = pipeline.script(
        script_path="tests/functional/pipelines/data/script.py"
    )
    script.inputs.create_param(code="transcripts", data_type=DataType.TEXT)
    script.inputs.create_param(code="speakers", data_type=DataType.LABEL)
    script.outputs.create_param(code="data", data_type=DataType.TEXT)

    input.outputs.input.link(segmentor.inputs.audio)
    segmentor.outputs.audio.link(speech_recognition.inputs.source_audio)
    segmentor.outputs.data.link(script.inputs.speakers)
    speech_recognition.outputs.data.link(script.inputs.transcripts)

    script.use_output("data")

    pipeline.save()

    output = pipeline.run(
        "s3://aixplain-platform-assets/samples/en/CPAC1x2.wav",
        version="2.0",
    )

    assert output["status"] == "SUCCESS"
    assert output.get("data") is not None
    assert len(output["data"]) > 0
    assert output["data"][0].get("segments") is not None
    assert len(output["data"][0]["segments"]) > 0


def test_decision_pipeline(pipeline):

    SENTIMENT_ANALYSIS_ASSET = "6172874f720b09325cbcdc33"

    input = pipeline.input()

    sentiment_analysis = pipeline.sentiment_analysis(
        asset_id=SENTIMENT_ANALYSIS_ASSET
    )

    positive_output = pipeline.output()
    negative_output = pipeline.output()
    decision_node = pipeline.decision(
        routes=[
            Route(
                type=RouteType.CHECK_VALUE,
                operation=Operation.EQUAL,
                value="POSITIVE",
                path=[positive_output],
            ),
            Route(
                type=RouteType.CHECK_VALUE,
                operation=Operation.DIFFERENT,
                value="POSITIVE",
                path=[negative_output],
            ),
        ]
    )

    input.outputs.input.link(sentiment_analysis.inputs.text)
    sentiment_analysis.outputs.data.link(decision_node.inputs.comparison)
    input.outputs.input.link(decision_node.inputs.passthrough)
    decision_node.outputs.input.link(positive_output.inputs.output)
    decision_node.outputs.input.link(negative_output.inputs.output)

    pipeline.save()

    output = pipeline.run("I feel so bad today!")

    assert output["status"] == "SUCCESS"
    assert output.get("data") is not None
    assert len(output["data"]) > 0
    assert output["data"][0].get("segments") is not None
    assert len(output["data"][0]["segments"]) > 0


def test_reconstructing_pipeline(pipeline):
    input = pipeline.input()

    segmentor = pipeline.speaker_diarization_audio(
        asset_id="62fab6ecb39cca09ca5bc365"
    )

    speech_recognition = pipeline.speech_recognition(
        asset_id="60ddefab8d38c51c5885ee38"
    )

    reconstructor = pipeline.bare_reconstructor()

    input.outputs.input.link(segmentor.inputs.audio)
    segmentor.outputs.audio.link(speech_recognition.inputs.source_audio)
    speech_recognition.outputs.data.link(reconstructor.inputs.data)

    reconstructor.use_output("data")

    pipeline.save()

    output = pipeline.run(
        "s3://aixplain-platform-assets/samples/en/CPAC1x2.wav",
    )
    assert output["status"] == "SUCCESS"
    assert output.get("data") is not None
    assert len(output["data"]) > 0
    assert output["data"][0].get("segments") is not None
    assert len(output["data"][0]["segments"]) > 0



__author__ = "aiXplain"

"""
Copyright 2023 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: aiXplain team
Date: September 25th 2023
Description:
    Supplier Enum
"""

import logging

from aixplain.utils import config
from aixplain.utils.file_utils import _request_with_retry
from enum import Enum
from urllib.parse import urljoin
import re


def clean_name(name):
    cleaned_name = re.sub(r"[ -]+", "_", name)
    cleaned_name = re.sub(r"[^a-zA-Z0-9_]", "", cleaned_name)
    cleaned_name = re.sub(r"^\d+", "", cleaned_name)
    return cleaned_name.upper()


def load_suppliers():
    api_key = config.TEAM_API_KEY
    aixplain_key = config.AIXPLAIN_API_KEY
    backend_url = config.BACKEND_URL

    url = urljoin(backend_url, "sdk/suppliers")
    if aixplain_key != "":
        api_key = aixplain_key
        headers = {"x-aixplain-key": aixplain_key, "Content-Type": "application/json"}
    else:
        headers = {"x-api-key": api_key, "Content-Type": "application/json"}
    r = _request_with_retry("get", url, headers=headers)
    if not 200 <= r.status_code < 300:
        raise Exception(
            f'Suppliers could not be loaded, probably due to the set API key (e.g. "{api_key}") is not valid. For help, please refer to the documentation (https://github.com/aixplain/aixplain#api-key-setup)'
        )
    resp = r.json()
    suppliers = Enum(
        "Supplier", {clean_name(w["name"]): {"id": w["id"], "name": w["name"], "code": w["code"]} for w in resp}, type=dict
    )

    return suppliers


Supplier = load_suppliers()



import argparse
import logging
from pydub import AudioSegment

FRAME_RATE = 16000

def create_compatible_audio(source_path, dest_path):
    """
    Function to resample an audio file and change the number of channels if there are more than 1.
    """
    # Load the audio file
    sound_file = AudioSegment.from_file(source_path)
    updated = False
    if sound_file.frame_rate != FRAME_RATE:
        # Resample the audio file
        logging.info(f'Resampling {sound_file.frame_rate} -> {FRAME_RATE}')
        sound_file = sound_file.set_frame_rate(FRAME_RATE)
        updated = True
    # If the audio file has more than one channel, convert it to mono
    if sound_file.channels > 1:
        logging.info(f'Changing no. channels {sound_file.channels} -> 1')
        sound_file = sound_file.set_channels(1)
        updated = True
    if updated:
        # Export the processed audio file
        sound_file.export(dest_path, format="wav")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Process some audio files.')
    parser.add_argument('--source_path', required=True, help='Source path for the audio file')
    parser.add_argument('--dest_path', required=True, help='Destination path for the processed audio file')

    args = parser.parse_args()

    create_compatible_audio(args.source_path, args.dest_path)



__author__ = "shreyassharma"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Duraikrishna Selvaraju, Thiago Castro Ferreira, Shreyas Sharma and Lucas Pavanelli
Date: October 25th 2022
Description:
    Metric Class
"""

from typing import Optional, Text, List, Union
from aixplain.modules.asset import Asset


class Metric(Asset):
    """Represents a metric to be computed on one or more peices of data. It is usually linked to a machine learning task.

    Attributes:
        id (Text): ID of the Metric
        name (Text): Name of the Metric
        description (Text): Description of the Metric
        supplier (Text, optional): author of the Metric. Defaults to "aiXplain".
        version (Text, optional): Metric version. Defaults to "1.0".
        additional_info: Any additional Metric info to be saved

    """

    def __init__(
        self,
        id: Text,
        name: Text,
        supplier: Text,
        is_reference_required: bool,
        is_source_required: bool,
        cost: float,
        function: Text,
        normalization_options: list = [],
        **additional_info,
    ) -> None:
        """Create a Metric with the necessary information

        Args:
            id (Text): ID of the Metric
            name (Text): Name of the Metric
            supplier (Text): author of the Metric
            is_reference_required (bool): does the metric use reference
            is_source_required (bool): does the metric use source
            cost (float): price of the metric
            normalization_options(list, [])
            **additional_info: Any additional Metric info to be saved
        """
        super().__init__(id, name, description="", supplier=supplier, version="1.0", cost=cost)
        self.is_source_required = is_source_required
        self.is_reference_required = is_reference_required
        self.normalization_options = normalization_options
        self.function = function
        self.additional_info = additional_info

    def __repr__(self) -> str:
        return f"<Metric {self.name}>"

    def add_normalization_options(self, normalization_options: List[str]):
        """Add a given set of normalization options to be used while benchmarking

        Args:
            normalization_options (List[str]): List of normalization options to be added
        """
        self.normalization_options.append(normalization_options)

    def run(
        self,
        hypothesis: Optional[Union[str, List[str]]] = None,
        source: Optional[Union[str, List[str]]] = None,
        reference: Optional[Union[str, List[str]]] = None,
    ):
        """Run the metric to calculate the scores.

        Args:
            hypothesis (Optional[Union[str, List[str]]], optional): Can give a single hypothesis or a list of hypothesis for metric calculation. Defaults to None.
            source (Optional[Union[str, List[str]]], optional): Can give a single source or a list of sources for metric calculation. Defaults to None.
            reference (Optional[Union[str, List[str]]], optional): Can give a single reference or a list of references for metric calculation. Defaults to None.
        """
        from aixplain.factories.model_factory import ModelFactory

        model = ModelFactory.get(self.id)
        payload = {
            "function": self.function,
            "supplier": self.supplier,
            "version": self.name,
        }
        if hypothesis is not None:
            if type(hypothesis) is str:
                hypothesis = [hypothesis]
            payload["hypotheses"] = hypothesis
        if self.is_source_required and source is not None:
            if type(source) is str:
                source = [source]
            payload["sources"] = source
        if self.is_reference_required and reference is not None:
            if type(reference) is str:
                reference = [[reference]]
            elif type(reference[0]) is str:
                reference = [[ref] for ref in reference]
            payload["references"] = reference
        return model.run(payload)



from typing import Union
from .base import Node, Link, Param


class LinkableMixin:
    """
    Linkable mixin class, this class will be used to link the output of the
    node to the input of another node.

    This class will be used to link the output of the node to the input of
    another node.
    """

    def link(
        self,
        to_node: Node,
        from_param: Union[str, Param],
        to_param: Union[str, Param],
    ) -> Link:
        """
        Link the output of the node to the input of another node. This method
        will link the output of the node to the input of another node.

        :param to_node: the node to link to the output
        :param from_param: the output parameter or the code of the output
        parameter
        :param to_param: the input parameter or the code of the input parameter
        :return: the link
        """
        return Link(
            pipeline=self.pipeline,
            from_node=self,
            to_node=to_node,
            from_param=from_param,
            to_param=to_param,
        )


class RoutableMixin:
    """
    Routable mixin class, this class will be used to route the input data to
    different nodes based on the input data type.
    """

    def route(self, *params: Param) -> Node:
        """
        Route the input data to different nodes based on the input data type.
        This method will automatically link the input data to the output data
        of the node.

        :param params: the output parameters
        :return: the router node
        """
        assert self.pipeline, "Node not attached to a pipeline"

        router = self.pipeline.router(
            [(param.data_type, param.node) for param in params]
        )
        self.outputs.input.link(router.inputs.input)
        for param in params:
            router.outputs.input.link(param)
        return router


class OutputableMixin:
    """
    Outputable mixin class, this class will be used to link the output of the
    node to the output node of the pipeline.
    """

    def use_output(self, param: Union[str, Param]) -> Node:
        """
        Use the output of the node as the output of the pipeline.
        This method will automatically link the output of the node to the
        output node of the pipeline.

        :param param: the output parameter or the code of the output parameter
        :return: the output node
        """
        assert self.pipeline, "Node not attached to a pipeline"
        output = self.pipeline.output()
        if isinstance(param, str):
            param = self.outputs[param]
        param.link(output.inputs.output)
        return output



from .data_split import DataSplit
from .data_subtype import DataSubtype
from .data_type import DataType
from .error_handler import ErrorHandler
from .file_type import FileType
from .function import Function, FunctionInputOutput
from .language import Language
from .license import License
from .onboard_status import OnboardStatus
from .ownership_type import OwnershipType
from .privacy import Privacy
from .storage_type import StorageType
from .supplier import Supplier
from .sort_by import SortBy
from .sort_order import SortOrder



__author__ = "shreyassharma"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Duraikrishna Selvaraju, Thiago Castro Ferreira, Shreyas Sharma and Lucas Pavanelli
Date: October 25th 2022
Description:
    Benchmark Class
"""
import logging
from typing import List, Text, Dict, Optional
from aixplain.utils import config
from aixplain.modules import Asset, Dataset, Metric, Model
from aixplain.modules.benchmark_job import BenchmarkJob
from urllib.parse import urljoin
import pandas as pd
from pathlib import Path
from aixplain.utils.file_utils import _request_with_retry, save_file


class Benchmark(Asset):
    """Benchmark is a powerful tool for benchmarking machine learning models and evaluating their performance on specific tasks.
    It represents a collection of Models, Datasets and Metrics to run associated Benchmark Jobs.

    Attributes:
        id (str): ID of the Benchmark.
        name (str): Name of the Benchmark.
        model_list (List[Model]): List of Models to be used for benchmarking.
        dataset_list (List[Dataset]): List of Datasets to be used for benchmarking.
        metric_list (List[Metric]): List of Metrics to be used for benchmarking.
        job_list (List[BenchmarkJob]): List of associated Benchmark Jobs.
        additional_info (dict): Any additional information to be saved with the Benchmark.
    """

    def __init__(
        self,
        id: Text,
        name: Text,
        dataset_list: List[Dataset],
        model_list: List[Model],
        metric_list: List[Metric],
        job_list: List[BenchmarkJob],
        description: Text = "",
        supplier: Text = "aiXplain",
        version: Text = "1.0",
        **additional_info
    ) -> None:
        """Create a Benchmark with the necessary information.

        Args:
            id (Text): ID of the Benchmark.
            name (Text): Name of the Benchmark.
            model_list (List[Model]): List of Models to be used for benchmarking
            dataset_list (List[Dataset]): List of Datasets to be used for benchmarking
            metric_list (List[Metric]): List of Metrics to be used for benchmarking
            job_list (List[BenchmarkJob]): List of associated Benchmark Jobs
            supplier (Text, optional): author of the Benchmark. Defaults to "aiXplain".
            version (Text, optional): Benchmark version. Defaults to "1.0".
            **additional_info: Any additional Benchmark info to be saved
        """
        super().__init__(id, name, description, supplier, version)
        self.model_list = model_list
        self.dataset_list = dataset_list
        self.metric_list = metric_list
        self.job_list = job_list
        self.additional_info = additional_info
        self.backend_url = config.BACKEND_URL
        self.api_key = config.TEAM_API_KEY
        self.aixplain_key = config.AIXPLAIN_API_KEY

    def __repr__(self) -> str:
        return f"<Benchmark {self.name}>"

    
    def start(self) -> BenchmarkJob:
        """Starts a new benchmark job(run)  for the current benchmark

        Returns:
            BenchmarkJob: Benchmark Job that just got started
        """
        benhchmark_id = None
        try:
            benhchmark_id = self.id
            url = urljoin(self.backend_url, f"sdk/benchmarks/{benhchmark_id}/start")
            headers = {"Authorization": f"Token {self.api_key}", "Content-Type": "application/json"}
            r = _request_with_retry("post", url, headers=headers)
            response = r.json()
            resp = BenchmarkJob._fetch_current_response(response["jobId"])
            logging.info(f"Starting Benchmark Job: Status for {benhchmark_id}: {resp}")
            return BenchmarkJob(resp["jobId"], resp["status"], resp["benchmark"]["id"])
        except Exception as e:
            error_message = f"Starting Benchmark Job: Error in Creating Benchmark {benhchmark_id} : {e}"
            logging.error(error_message, exc_info=True)
            return None


__author__ = "aixplain"

from aixplain.factories import WalletFactory
import aixplain.utils.config as config
import requests_mock


def test_wallet_service():
    with requests_mock.Mocker() as mock:
        url = f"{config.BACKEND_URL}/sdk/billing/wallet"
        headers = {"x-api-key": config.TEAM_API_KEY, "Content-Type": "application/json"}
        ref_response = {"totalBalance": 5, "reservedBalance": "0"}
        mock.get(url, headers=headers, json=ref_response)
        wallet = WalletFactory.get()
    assert wallet.total_balance == float(ref_response["totalBalance"])
    assert wallet.reserved_balance == float(ref_response["reservedBalance"])
    assert wallet.available_balance == float(ref_response["totalBalance"]) - float(ref_response["reservedBalance"])



from enum import Enum


class RouteType(str, Enum):
    CHECK_TYPE = "checkType"
    CHECK_VALUE = "checkValue"


class Operation(str, Enum):
    GREATER_THAN = "greaterThan"
    GREATER_THAN_OR_EQUAL = "greaterThanOrEqual"
    LESS_THAN = "lessThan"
    LESS_THAN_OR_EQUAL = "lessThanOrEqual"
    EQUAL = "equal"
    DIFFERENT = "different"
    CONTAIN = "contain"
    NOT_CONTAIN = "notContain"


class NodeType(str, Enum):
    ASSET = "ASSET"
    INPUT = "INPUT"
    OUTPUT = "OUTPUT"
    SCRIPT = "SCRIPT"
    SEGMENTOR = "SEGMENT"
    RECONSTRUCTOR = "RECONSTRUCT"
    ROUTER = "ROUTER"
    DECISION = "DECISION"


class AssetType(str, Enum):
    MODEL = "MODEL"


class FunctionType(str, Enum):
    AI = "AI"
    SEGMENTOR = "SEGMENTOR"
    RECONSTRUCTOR = "RECONSTRUCTOR"


class ParamType:
    INPUT = "INPUT"
    OUTPUT = "OUTPUT"



__author__ = "michaellam"
from pathlib import Path
import json
from aixplain.factories.model_factory import ModelFactory
from tests.test_utils import delete_asset, delete_service_account
from aixplain.utils import config
import docker
import pytest

def test_login():
    response = ModelFactory.asset_repo_login()
    assert response["username"] == "AWS"
    assert response["registry"] == "535945872701.dkr.ecr.us-east-1.amazonaws.com"
    assert "password" in response.keys()

    # Test cleanup
    delete_service_account(config.TEAM_API_KEY)


def test_create_asset_repo():
    with open(Path("tests/test_requests/create_asset_request.json")) as f:
        mock_register_payload = json.load(f)
    name = mock_register_payload["name"]
    description = mock_register_payload["description"]
    function = mock_register_payload["function"]
    source_language = mock_register_payload["sourceLanguage"]
    input_modality = mock_register_payload["input_modality"]
    output_modality = mock_register_payload["output_modality"]
    documentation_url = mock_register_payload["documentation_url"]
    response = ModelFactory.create_asset_repo(name, description, function, source_language, input_modality, output_modality, documentation_url, config.TEAM_API_KEY)
    response_dict = dict(response)
    assert "id" in response_dict.keys()
    assert "repositoryName" in response_dict.keys()

    # Test cleanup
    delete_asset(response["id"], config.TEAM_API_KEY)


def test_list_host_machines():
    response = ModelFactory.list_host_machines()
    for hosting_machine_dict in response:
        assert "code" in hosting_machine_dict.keys()
        assert "type" in hosting_machine_dict.keys()
        assert "cores" in hosting_machine_dict.keys()
        assert "memory" in hosting_machine_dict.keys()
        assert "hourlyCost" in hosting_machine_dict.keys()


def test_get_functions():
    # Verbose
    response = ModelFactory.list_functions(True)
    items = response["items"]
    for item in items:
        assert "output" in item.keys()
        assert "params" in item.keys()
        assert "id" in item.keys()
        assert "name" in item.keys()

    # Non-verbose
    response = ModelFactory.list_functions()  # Not verbose by default
    items = response["items"]
    for item in items:
        assert "output" not in item.keys()
        assert "params" not in item.keys()
        assert "id" not in item.keys()
        assert "name" in item.keys()


@pytest.mark.skip(reason="Not included in first release")
def list_image_repo_tags():
    response = ModelFactory.list_image_repo_tags()
    assert "Image tags" in response.keys()
    assert "nextToken" in response.keys()



__author__ = "aiXplain"

"""
Copyright 2024 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Lucas Pavanelli and Thiago Castro Ferreira
Date: May 16th 2024
Description:
    Agentification Class
"""
from typing import Text, Optional

from aixplain.enums.function import Function
from aixplain.enums.supplier import Supplier


class Tool:
    """Specialized software or resource designed to assist the AI in executing specific tasks or functions based on user commands.

    Attributes:
        name (Text): name of the tool
        description (Text): descriptiion of the tool
        function (Function): task that the tool performs
        supplier (Optional[Union[Dict, Text, Supplier, int]], optional): Preferred supplier to perform the task. Defaults to None.
    """

    def __init__(
        self,
        name: Text,
        description: Text,
        function: Function,
        supplier: Optional[Supplier] = None,
        **additional_info,
    ) -> None:
        """Specialized software or resource designed to assist the AI in executing specific tasks or functions based on user commands.

        Args:
            name (Text): name of the tool
            description (Text): descriptiion of the tool
            function (Function): task that the tool performs
            supplier (Optional[Union[Dict, Text, Supplier, int]], optional): Preferred supplier to perform the task. Defaults to None.
        """
        self.name = name
        self.description = description
        self.function = function
        self.supplier = supplier
        self.additional_info = additional_info



__author__ = "lucaspavanelli"

"""
Copyright 2024 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Thiago Castro Ferreira, Shreyas Sharma and Lucas Pavanelli
Date: June 4th 2024
Description:
    Large Language Model Class
"""
import time
import json
import logging
import traceback
from aixplain.enums import Function, Supplier
from aixplain.modules.model import Model
from aixplain.utils import config
from aixplain.utils.file_utils import _request_with_retry
from typing import Union, Optional, List, Text, Dict


class LLM(Model):
    """Ready-to-use LLM model. This model can be run in both synchronous and asynchronous manner.

    Attributes:
        id (Text): ID of the Model
        name (Text): Name of the Model
        description (Text, optional): description of the model. Defaults to "".
        api_key (Text, optional): API key of the Model. Defaults to None.
        url (Text, optional): endpoint of the model. Defaults to config.MODELS_RUN_URL.
        supplier (Union[Dict, Text, Supplier, int], optional): supplier of the asset. Defaults to "aiXplain".
        version (Text, optional): version of the model. Defaults to "1.0".
        function (Text, optional): model AI function. Defaults to None.
        url (str): URL to run the model.
        backend_url (str): URL of the backend.
        pricing (Dict, optional): model price. Defaults to None.
        **additional_info: Any additional Model info to be saved
    """

    def __init__(
        self,
        id: Text,
        name: Text,
        description: Text = "",
        api_key: Optional[Text] = None,
        supplier: Union[Dict, Text, Supplier, int] = "aiXplain",
        version: Optional[Text] = None,
        function: Optional[Function] = None,
        is_subscribed: bool = False,
        cost: Optional[Dict] = None,
        **additional_info,
    ) -> None:
        """LLM Init

        Args:
            id (Text): ID of the Model
            name (Text): Name of the Model
            description (Text, optional): description of the model. Defaults to "".
            api_key (Text, optional): API key of the Model. Defaults to None.
            supplier (Union[Dict, Text, Supplier, int], optional): supplier of the asset. Defaults to "aiXplain".
            version (Text, optional): version of the model. Defaults to "1.0".
            function (Function, optional): model AI function. Defaults to None.
            is_subscribed (bool, optional): Is the user subscribed. Defaults to False.
            cost (Dict, optional): model price. Defaults to None.
            **additional_info: Any additional Model info to be saved
        """
        assert function == Function.TEXT_GENERATION, "LLM only supports large language models (i.e. text generation function)"
        super().__init__(
            id=id,
            name=name,
            description=description,
            supplier=supplier,
            version=version,
            cost=cost,
            function=function,
            is_subscribed=is_subscribed,
            api_key=api_key,
            **additional_info,
        )
        self.url = config.MODELS_RUN_URL
        self.backend_url = config.BACKEND_URL

    def run(
        self,
        data: Text,
        context: Optional[Text] = None,
        prompt: Optional[Text] = None,
        history: Optional[List[Dict]] = None,
        temperature: float = 0.001,
        max_tokens: int = 128,
        top_p: float = 1.0,
        name: Text = "model_process",
        timeout: float = 300,
        parameters: Dict = {},
        wait_time: float = 0.5,
    ) -> Dict:
        """Synchronously running a Large Language Model (LLM) model.

        Args:
            data (Union[Text, Dict]): Text to LLM or last user utterance of a conversation.
            context (Optional[Text], optional): System message. Defaults to None.
            prompt (Optional[Text], optional): Prompt Message which comes on the left side of the last utterance. Defaults to None.
            history (Optional[List[Dict]], optional): Conversation history in OpenAI format ([{ "role": "assistant", "content": "Hello, world!"}]). Defaults to None.
            temperature (float, optional): LLM temperature. Defaults to 0.001.
            max_tokens (int, optional): Maximum Generation Tokens. Defaults to 128.
            top_p (float, optional): Top P. Defaults to 1.0.
            name (Text, optional): ID given to a call. Defaults to "model_process".
            timeout (float, optional): total polling time. Defaults to 300.
            parameters (Dict, optional): optional parameters to the model. Defaults to "{}".
            wait_time (float, optional): wait time in seconds between polling calls. Defaults to 0.5.

        Returns:
            Dict: parsed output from model
        """
        start = time.time()
        try:
            response = self.run_async(
                data,
                name=name,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                context=context,
                prompt=prompt,
                history=history,
                parameters=parameters,
            )
            if response["status"] == "FAILED":
                end = time.time()
                response["elapsed_time"] = end - start
                return response
            poll_url = response["url"]
            end = time.time()
            response = self.sync_poll(poll_url, name=name, timeout=timeout, wait_time=wait_time)
            return response
        except Exception as e:
            msg = f"Error in request for {name} - {traceback.format_exc()}"
            logging.error(f"LLM Run: Error in running for {name}: {e}")
            end = time.time()
            return {"status": "FAILED", "error": msg, "elapsed_time": end - start}

    def run_async(
        self,
        data: Text,
        context: Optional[Text] = None,
        prompt: Optional[Text] = None,
        history: Optional[List[Dict]] = None,
        temperature: float = 0.001,
        max_tokens: int = 128,
        top_p: float = 1.0,
        name: Text = "model_process",
        parameters: Dict = {},
    ) -> Dict:
        """Runs asynchronously a model call.

        Args:
            data (Union[Text, Dict]): Text to LLM or last user utterance of a conversation.
            context (Optional[Text], optional): System message. Defaults to None.
            prompt (Optional[Text], optional): Prompt Message which comes on the left side of the last utterance. Defaults to None.
            history (Optional[List[Dict]], optional): Conversation history in OpenAI format ([{ "role": "assistant", "content": "Hello, world!"}]). Defaults to None.
            temperature (float, optional): LLM temperature. Defaults to 0.001.
            max_tokens (int, optional): Maximum Generation Tokens. Defaults to 128.
            top_p (float, optional): Top P. Defaults to 1.0.
            name (Text, optional): ID given to a call. Defaults to "model_process".
            parameters (Dict, optional): optional parameters to the model. Defaults to "{}".

        Returns:
            dict: polling URL in response
        """
        headers = {"x-api-key": self.api_key, "Content-Type": "application/json"}

        from aixplain.factories.file_factory import FileFactory

        data = FileFactory.to_link(data)
        if isinstance(data, dict):
            payload = data
        else:
            try:
                payload = json.loads(data)
                if isinstance(payload, dict) is False:
                    if isinstance(payload, int) is True or isinstance(payload, float) is True:
                        payload = str(payload)
                    payload = {"data": payload}
            except Exception:
                payload = {"data": data}
        parameters.update(
            {
                "context": payload["context"] if "context" in payload else context,
                "prompt": payload["prompt"] if "prompt" in payload else prompt,
                "history": payload["history"] if "history" in payload else history,
                "temperature": payload["temperature"] if "temperature" in payload else temperature,
                "max_tokens": payload["max_tokens"] if "max_tokens" in payload else max_tokens,
                "top_p": payload["top_p"] if "top_p" in payload else top_p,
            }
        )
        payload.update(parameters)
        payload = json.dumps(payload)

        call_url = f"{self.url}/{self.id}"
        r = _request_with_retry("post", call_url, headers=headers, data=payload)
        logging.info(f"Model Run Async: Start service for {name} - {self.url} - {payload} - {headers}")

        resp = None
        try:
            if 200 <= r.status_code < 300:
                resp = r.json()
                logging.info(f"Result of request for {name} - {r.status_code} - {resp}")
                poll_url = resp["data"]
                response = {"status": "IN_PROGRESS", "url": poll_url}
            else:
                if r.status_code == 401:
                    error = "Unauthorized API key: Please verify the spelling of the API key and its current validity."
                elif 460 <= r.status_code < 470:
                    error = "Subscription-related error: Please ensure that your subscription is active and has not expired."
                elif 470 <= r.status_code < 480:
                    error = "Billing-related error: Please ensure you have enough credits to run this model. "
                elif 480 <= r.status_code < 490:
                    error = "Supplier-related error: Please ensure that the selected supplier provides the model you are trying to access."
                elif 490 <= r.status_code < 500:
                    error = "Validation-related error: Please ensure all required fields are provided and correctly formatted."
                else:
                    status_code = str(r.status_code)
                    error = f"Status {status_code}: Unspecified error: An unspecified error occurred while processing your request."
                response = {"status": "FAILED", "error_message": error}
                logging.error(f"Error in request for {name} - {r.status_code}: {error}")               
        except Exception:
            response = {"status": "FAILED"}
            msg = f"Error in request for {name} - {traceback.format_exc()}"
            logging.error(f"Model Run Async: Error in running for {name}: {resp}")
            if resp is not None:
                response["error"] = msg
        return response



__author__ = "aixplain"

"""
Copyright 2024 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: aiXplain Team
Date: August 20th 2024
Description:
    Wallet Class
"""


class Wallet:
    def __init__(self, total_balance: float, reserved_balance: float):
        """
        Args:
            total_balance (float): total credit balance
            reserved_balance (float): reserved credit balance
            available_balance (float): available balance (total - credit)
        """
        self.total_balance = total_balance
        self.reserved_balance = reserved_balance
        self.available_balance =  total_balance-reserved_balance


__author__ = "thiagocastroferreira"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import json
import pytest
from aixplain.factories import PipelineFactory
from aixplain.modules import Pipeline
from uuid import uuid4


def test_create_pipeline_from_json():
    pipeline_json = "tests/functional/pipelines/data/pipeline.json"
    pipeline_name = str(uuid4())
    pipeline = PipelineFactory.create(name=pipeline_name, pipeline=pipeline_json)

    assert isinstance(pipeline, Pipeline)
    assert pipeline.id != ""
    pipeline.delete()


def test_create_pipeline_from_string():
    pipeline_json = "tests/functional/pipelines/data/pipeline.json"
    with open(pipeline_json) as f:
        pipeline_dict = json.load(f)

    pipeline_name = str(uuid4())
    pipeline = PipelineFactory.create(name=pipeline_name, pipeline=pipeline_dict)

    assert isinstance(pipeline, Pipeline)
    assert pipeline.id != ""
    pipeline.delete()


def test_update_pipeline():
    pipeline_json = "tests/functional/pipelines/data/pipeline.json"
    with open(pipeline_json) as f:
        pipeline_dict = json.load(f)

    pipeline_name = str(uuid4())
    pipeline = PipelineFactory.create(name=pipeline_name, pipeline=pipeline_dict)

    pipeline.update(pipeline=pipeline_json, save_as_asset=True)
    assert isinstance(pipeline, Pipeline)
    assert pipeline.id != ""
    pipeline.delete()


def test_create_pipeline_wrong_path():
    pipeline_name = str(uuid4())

    with pytest.raises(Exception):
        PipelineFactory.create(name=pipeline_name, pipeline="/")



import logging
from typing import List, Text, Dict, Optional
from aixplain.utils import config
from aixplain.modules import Asset, Dataset, Metric, Model
from urllib.parse import urljoin
import pandas as pd
from pathlib import Path
from aixplain.utils.file_utils import _request_with_retry, save_file

class BenchmarkJob:
    """Benchmark Job Represents a single run of an already created Benchmark.

    Attributes:
        id (str): ID of the Benchmark Job.
        status (str): Status of the Benchmark Job.
        benchmark_id (str): ID of the associated parent Benchmark.
        additional_info (dict): Any additional information to be saved with the Benchmark Job.

    """

    def __init__(self, id: Text, status: Text, benchmark_id: Text, **additional_info) -> None:
        """Create a Benchmark Job with the necessary information. Each Job is a run of a parent Benchmark

        Args:
            id (Text): ID of the Benchmark Job
            status (Text): Status of the Benchmark Job
            benchmark_id (Text): ID of the associated parent Benchmark
            **additional_info: Any additional Benchmark Job info to be saved
        """
        self.id = id
        self.status = status
        self.benchmark_id = benchmark_id
        self.additional_info = additional_info

    @classmethod
    def _create_benchmark_job_from_response(cls, response: Dict):
        return BenchmarkJob(response["jobId"], response["status"], response["benchmark"]["id"])
    
    @classmethod
    def _fetch_current_response(cls, job_id: Text) -> dict:
        url = urljoin(config.BACKEND_URL, f"sdk/benchmarks/jobs/{job_id}")
        if  config.AIXPLAIN_API_KEY != "":
            headers = {"x-aixplain-key": f"{config.AIXPLAIN_API_KEY}", "Content-Type": "application/json"}
        else:
            headers = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
        r = _request_with_retry("get", url, headers=headers)
        resp = r.json()
        return resp
    
    def _update_from_response(self, response: dict):
        self.status = response['status']

    def __repr__(self) -> str:
        return f"<Benchmark Job({self.id})>"
    
    def check_status(self):
        response = self._fetch_current_response(self.id)
        self._update_from_response(response)
        return self.status
    
    def download_results_as_csv(self, save_path: Optional[Text] = None, return_dataframe: bool = False):
        """Get the results of the benchmark job in a CSV format.
        The results can either be downloaded locally or returned in the form of pandas.DataFrame.


        Args:
            save_path (Text, optional): Path to save the CSV if return_dataframe is False. If None, a ranmdom path is generated. defaults to None.
            return_dataframe (bool): If True, the result is returned as pandas.DataFrame else saved as a CSV file. defaults to False.

        Returns:
            str/pandas.DataFrame: results as path of locally saved file if return_dataframe is False else as a pandas dataframe
        """
        try:
            resp = self._fetch_current_response(self.id)
            logging.info(f"Downloading Benchmark Results: Status of downloading results for {self.id}: {resp}")
            if "reportUrl" not in resp or resp['reportUrl'] == "":
                logging.error(
                    f"Downloading Benchmark Results: Can't get download results as they aren't generated yet. Please wait for a while."
                )
                return None
            csv_url = resp["reportUrl"]
            if return_dataframe:
                downloaded_path = save_file(csv_url, save_path)
                df = pd.read_csv(downloaded_path)
                if save_path is None:
                    Path(downloaded_path).unlink()
                return df
            else:
                downloaded_path = save_file(csv_url, save_path)
                return downloaded_path
        except Exception as e:
            error_message = f"Downloading Benchmark Results: Error in Downloading Benchmark Results : {e}"
            logging.error(error_message, exc_info=True)
            raise Exception(error_message)
        
    def __simplify_scores(self, scores):
        simplified_score_list  = []
        for model_id, model_info in scores.items():
            model_scores = model_info["rawScores"]
            # model = Mode
            row = {"Model": model_id}
            for score_info in model_scores:
                row[score_info["longName"]] = score_info["average"]
            simplified_score_list.append(row)
        return simplified_score_list




    def get_scores(self, return_simplified=True, return_as_dataframe=True):
        try:
            resp = self._fetch_current_response(self.id)
            iterations = resp.get("iterations", [])
            scores = {}
            for iteration_info in iterations:
                model_id = iteration_info["pipeline"]
                model_info = {
                    "creditsUsed" : round(iteration_info.get("credits", 0),5),
                    "timeSpent" : round(iteration_info.get("runtime", 0),2),
                    "status" : iteration_info["status"],
                    "rawScores" : iteration_info["scores"],
                }
                scores[model_id] = model_info
            
            if return_simplified:
                simplified_scores = self.__simplify_scores(scores)
                if return_as_dataframe:
                    simplified_scores = pd.DataFrame(simplified_scores)
                return simplified_scores
            else:
                return scores
        except Exception as e:
            error_message = f"Benchmark scores: Error in Getting benchmark scores: {e}"
            logging.error(error_message, exc_info=True)
            raise Exception(error_message)
        
    
    def get_failuire_rate(self, return_as_dataframe=True):
        try:
            scores = self.get_scores(return_simplified=False)
            failure_rates = {}
            for model_id, model_info in scores.items():
                if len(model_info["rawScores"]) == 0:
                    failure_rates[model_id] = 0
                    continue
                score_info = model_info["rawScores"][0] 
                num_succesful = score_info["count"]
                num_failed = score_info["failedSegmentsCount"]
                failuire_rate =  (num_failed * 100) / (num_succesful+num_failed)
                failure_rates[model_id] = failuire_rate
            if return_as_dataframe:
                df = pd.DataFrame()
                df["Model"] = list(failure_rates.keys())
                df["Failuire Rate"] = list(failure_rates.values())
                return df
            else:
                return failure_rates
        except Exception as e:
            error_message = f"Benchmark scores: Error in Getting benchmark failuire rate: {e}"
            logging.error(error_message, exc_info=True)
            raise Exception(error_message)
        
    def get_all_explanations(self):
        try:
            resp = self._fetch_current_response(self)
            raw_explanations = resp.get("explanation", {})
            if "metricInDependent" not in raw_explanations:
                raw_explanations["metricInDependent"] = []
            if "metricDependent" not in raw_explanations:
                raw_explanations["metricDependent"] = []
            return raw_explanations
        except Exception as e:
            error_message = f"Benchmark scores: Error in Getting benchmark explanations: {e}"
            logging.error(error_message, exc_info=True)
            raise Exception(error_message)
    
    def get_localized_explanations(self, metric_dependant: bool, group_by_task: bool = False):
        try:
            raw_explanations = self.get_all_explanations()
            if metric_dependant:
                localized_explanations = raw_explanations["metricDependent"]
                if len(localized_explanations) == 0:
                    localized_explanations = {}
                else:
                    grouped_explanations = {}
                    task_list = []
                    first_explanation = localized_explanations[0]
                    for task in first_explanation:
                        if task not in ["scoreId", "datasetId"]:
                            task_list.append(task)

                    if group_by_task:
                        for task in task_list:
                            task_explanation = {}
                            for explanation_item in localized_explanations:
                                item_task_explanation = explanation_item[task]
                                identifier = explanation_item["scoreId"]
                                task_explanation[identifier] = item_task_explanation
                            grouped_explanations[task] = task_explanation
                    else:
                        for explanation_item in localized_explanations:
                            identifier = explanation_item["scoreId"]
                            grouped_explanations[identifier] = explanation_item
                    localized_explanations = grouped_explanations
            else:
                localized_explanations = raw_explanations["metricInDependent"]
                if len(localized_explanations) == 0:
                    localized_explanations =  {}
                else:
                    localized_explanations = localized_explanations[0]
            return localized_explanations

        except Exception as e:
            error_message = f"Benchmark scores: Error in Getting benchmark explanations: {e}"
            logging.error(error_message, exc_info=True)
            raise Exception(error_message)


__author__ = "aiXplain"

"""
Copyright 2023 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: aiXplain team
Date: March 20th 2023
Description:
    License Enum
"""

import logging

from aixplain.utils import config
from aixplain.utils.request_utils import _request_with_retry
from enum import Enum
from urllib.parse import urljoin


def load_licenses():
    try:
        api_key = config.TEAM_API_KEY
        aixplain_key = config.AIXPLAIN_API_KEY
        backend_url = config.BACKEND_URL

        url = urljoin(backend_url, "sdk/licenses")
        if aixplain_key != "":
            api_key = aixplain_key
            headers = {"x-aixplain-key": aixplain_key, "Content-Type": "application/json"}
        else:
            headers = {"x-api-key": api_key, "Content-Type": "application/json"}
        r = _request_with_retry("get", url, headers=headers)
        if not 200 <= r.status_code < 300:
            raise Exception(
                f'Licenses could not be loaded, probably due to the set API key (e.g. "{api_key}") is not valid. For help, please refer to the documentation (https://github.com/aixplain/aixplain#api-key-setup)'
            )
        resp = r.json()
        return Enum("License", {"_".join(w["name"].split()): w["id"] for w in resp}, type=str)
    except Exception as e:
        logging.exception("License Loading Error")
        raise Exception("License Loading Error")


License = load_licenses()



__author__ = "shreyassharma"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Duraikrishna Selvaraju, Thiago Castro Ferreira, Shreyas Sharma and Lucas Pavanelli
Date: December 2nd 2022
Description:
    Benchmark Factory Class
"""

import logging
from typing import Dict, List, Optional, Text
import json
import pandas as pd
from pathlib import Path
from aixplain.enums.supplier import Supplier
from aixplain.modules import Dataset, Metric, Model
from aixplain.modules.benchmark_job import BenchmarkJob
from aixplain.modules.benchmark import Benchmark
from aixplain.factories.metric_factory import MetricFactory
from aixplain.factories.dataset_factory import DatasetFactory
from aixplain.factories.model_factory import ModelFactory
from aixplain.utils import config
from aixplain.utils.file_utils import _request_with_retry, save_file
from urllib.parse import urljoin
from warnings import warn


class BenchmarkFactory:
    """A static class for creating and managing the Benchmarking experience.

    Attributes:
        backend_url (str): The URL for the backend.
    """

    aixplain_key = config.AIXPLAIN_API_KEY
    backend_url = config.BACKEND_URL

    @classmethod
    def _create_benchmark_job_from_response(cls, response: Dict) -> BenchmarkJob:
        """Converts response Json to 'BenchmarkJob' object

        Args:
            response (Dict): Json from API

        Returns:
            BenchmarkJob: Coverted 'BenchmarkJob' object
        """
        return BenchmarkJob(response["jobId"], response["status"], response["benchmark"]["id"])

    @classmethod
    def _get_benchmark_jobs_from_benchmark_id(cls, benchmark_id: Text) -> List[BenchmarkJob]:
        """Get list of benchmark jobs from benchmark id

        Args:
            benchmark_id (Text): ID of benchmark

        Returns:
            List[BenchmarkJob]: List of associated benchmark jobs
        """
        url = urljoin(cls.backend_url, f"sdk/benchmarks/{benchmark_id}/jobs")
        if cls.aixplain_key != "":
            headers = {"x-aixplain-key": f"{cls.aixplain_key}", "Content-Type": "application/json"}
        else:
            headers = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
        r = _request_with_retry("get", url, headers=headers)
        resp = r.json()
        job_list = [cls._create_benchmark_job_from_response(job_info) for job_info in resp]
        return job_list

    @classmethod
    def _create_benchmark_from_response(cls, response: Dict) -> Benchmark:
        """Converts response Json to 'Benchmark' object

        Args:
            response (Dict): Json from API

        Returns:
            Benchmark: Coverted 'Benchmark' object
        """
        model_list = [ModelFactory().get(model_info["id"]) for model_info in response["model"]]
        dataset_list = [DatasetFactory().get(dataset_id) for dataset_id in response["datasets"]]
        metric_list = [MetricFactory().get(metric_info["id"]) for metric_info in response["metrics"]]
        job_list = cls._get_benchmark_jobs_from_benchmark_id(response["id"])
        return Benchmark(response["id"], response["name"], dataset_list, model_list, metric_list, job_list)

    @classmethod
    def get(cls, benchmark_id: str) -> Benchmark:
        """Create a 'Benchmark' object from Benchmark id

        Args:
            benchmark_id (Text): Benchmark ID of required Benchmark.

        Returns:
            Benchmark: Created 'Benchmark' object
        """
        resp = None
        try:
            url = urljoin(cls.backend_url, f"sdk/benchmarks/{benchmark_id}")
            if cls.aixplain_key != "":
                headers = {"x-aixplain-key": f"{cls.aixplain_key}", "Content-Type": "application/json"}
            else:
                headers = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
            logging.info(f"Start service for GET Benchmark  - {url} - {headers}")
            r = _request_with_retry("get", url, headers=headers)
            resp = r.json()
            benchmark = cls._create_benchmark_from_response(resp)
        except Exception as e:
            status_code = 400
            if resp is not None and "statusCode" in resp:
                status_code = resp["statusCode"]
                message = resp["message"]
                message = f"Benchmark Creation: Status {status_code} - {message}"
            else:
                message = f"Benchmark Creation: Unspecified Error"
            logging.error(f"Benchmark Creation Failed: {e}")
            raise Exception(f"Status {status_code}: {message}")
        return benchmark

    @classmethod
    def get_job(cls, job_id: Text) -> BenchmarkJob:
        """Create a 'BenchmarkJob' object from job id

        Args:
            job_id (Text): ID of the required BenchmarkJob.

        Returns:
            BenchmarkJob: Created 'BenchmarkJob' object
        """
        url = urljoin(cls.backend_url, f"sdk/benchmarks/jobs/{job_id}")
        if cls.aixplain_key != "":
            headers = {"x-aixplain-key": f"{cls.aixplain_key}", "Content-Type": "application/json"}
        else:
            headers = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
        r = _request_with_retry("get", url, headers=headers)
        resp = r.json()
        benchmarkJob = cls._create_benchmark_job_from_response(resp)
        return benchmarkJob

    @classmethod
    def _validate_create_benchmark_payload(cls, payload):
        if len(payload["datasets"]) != 1:
            raise Exception("Please use exactly one dataset")
        if len(payload["metrics"]) == 0:
            raise Exception("Please use exactly one metric")
        if len(payload["model"]) == 0:
            raise Exception("Please use exactly one model")
        clean_metrics_info = {}
        for metric_info in payload["metrics"]:
            metric_id = metric_info["id"]
            if metric_id not in clean_metrics_info:
                clean_metrics_info[metric_id] = metric_info["configurations"]
            else:
                clean_metrics_info[metric_id] += metric_info["configurations"]
            clean_metrics_info[metric_id] = list(set(clean_metrics_info[metric_id]))
            if len(clean_metrics_info[metric_id]) == 0:
                clean_metrics_info[metric_id] = [[]]
        payload["metrics"] = [
            {"id": metric_id, "configurations": metric_config} for metric_id, metric_config in clean_metrics_info.items()
        ]
        return payload

    @classmethod
    def create(cls, name: str, dataset_list: List[Dataset], model_list: List[Model], metric_list: List[Metric]) -> Benchmark:
        """Creates a benchmark based on the information provided like name, dataset list, model list and score list.
        Note: This only creates a benchmark. It needs to run seperately using start_benchmark_job.

        Args:
            name (str): Unique Name of benchmark
            dataset_list (List[Dataset]): List of Datasets to be used for benchmarking
            model_list (List[Model]): List of Models to be used for benchmarking
            metric_list (List[Metric]): List of Metrics to be used for benchmarking

        Returns:
            Benchmark: _description_
        """
        payload = {}
        try:
            url = urljoin(cls.backend_url, f"sdk/benchmarks")
            headers = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
            payload = {
                "name": name,
                "datasets": [dataset.id for dataset in dataset_list],
                "model": [model.id for model in model_list],
                "metrics": [{"id": metric.id, "configurations": metric.normalization_options} for metric in metric_list],
                "shapScores": [],
                "humanEvaluationReport": False,
                "automodeTraining": False,
            }
            clean_payload = cls._validate_create_benchmark_payload(payload)
            payload = json.dumps(clean_payload)
            r = _request_with_retry("post", url, headers=headers, data=payload)
            resp = r.json()
            logging.info(f"Creating Benchmark Job: Status for {name}: {resp}")
            return cls.get(resp["id"])
        except Exception as e:
            error_message = f"Creating Benchmark Job: Error in Creating Benchmark with payload {payload} : {e}"
            logging.error(error_message, exc_info=True)
            return None

    @classmethod
    def list_normalization_options(cls, metric: Metric, model: Model) -> List[str]:
        """Get list of supported normalization options for a metric and model to be used in benchmarking

        Args:
            metric (Metric): Metric for which normalization options are to be listed
            model(Model): Model to be used in benchmarking

        Returns:
            List[str]: List of supported normalization options
        """
        try:
            url = urljoin(cls.backend_url, f"sdk/benchmarks/normalization-options")
            if cls.aixplain_key != "":
                headers = {"x-aixplain-key": f"{cls.aixplain_key}", "Content-Type": "application/json"}
            else:
                headers = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
            payload = json.dumps({"metricId": metric.id, "modelIds": [model.id]})
            r = _request_with_retry("post", url, headers=headers, data=payload)
            resp = r.json()
            logging.info(f"Listing Normalization Options: Status of listing options: {resp}")
            normalization_options = [item["value"] for item in resp]
            return normalization_options
        except Exception as e:
            error_message = f"Listing Normalization Options: Error in getting Normalization Options: {e}"
            logging.error(error_message, exc_info=True)
            return []

    @classmethod
    def get_benchmark_job_scores(cls, job_id):
        def __get_model_name(model_id):
            model = ModelFactory.get(model_id)
            supplier = str(model.supplier)
            try:
                if isinstance(supplier, Supplier):
                    name = f"{supplier.name}"
                else:
                    name = f"{eval(supplier)['name']}"
            except Exception as e:
                logging.error(f"{e}")
                name = f"{supplier}"
            if model.version is not None:
                name = f"{name}({model.version})"
            return name
        benchmarkJob = cls.get_job(job_id)
        scores_df = benchmarkJob.get_scores()
        scores_df["Model"] = scores_df["Model"].apply(lambda x: __get_model_name(x))
        return scores_df


__author__ = "lucaspavanelli"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Duraikrishna Selvaraju, Thiago Castro Ferreira, Shreyas Sharma and Lucas Pavanelli
Date: June 14th 2023
Description:
    FinetuneCost Class
"""

from typing import Dict


class FinetuneCost:
    def __init__(
        self,
        training: Dict,
        inference: Dict,
        hosting: Dict,
    ) -> None:
        """Create a FinetuneCost object with training, inference, and hosting cost information.

        Args:
            training (Dict): Dictionary containing training cost information.
            inference (Dict): Dictionary containing inference cost information.
            hosting (Dict): Dictionary containing hosting cost information.
        """
        self.training = training
        self.inference = inference
        self.hosting = hosting

    def to_dict(self) -> Dict:
        """Convert the FinetuneCost object to a dictionary.

        Returns:
            Dict: A dictionary representation of the FinetuneCost object.
        """
        return {"trainingCost": self.training, "inferenceCost": self.inference, "hostingCost": self.hosting}



__author__ = "aiXplain"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: aiXplain team
Date: March 20th 2023
Description:
    File Class
"""

import pathlib

from aixplain.enums.data_split import DataSplit
from aixplain.enums.file_type import FileType
from typing import Optional, Text, Union


class File:
    def __init__(
        self,
        path: Union[Text, pathlib.Path],
        extension: Union[Text, FileType],
        data_split: Optional[DataSplit] = None,
        compression: Optional[Text] = None,
    ) -> None:
        """File Class

        Description:
            File where samples of a data is stored in

        Args:
            path (Union[Text, pathlib.Path]): File path
            extension (Union[Text, FileType]): File extension (e.g. CSV, TXT, etc.)
            data_split (Optional[DataSplit], optional): Data split of the file. Defaults to None.
            compression (Optional[Text], optional): Compression extension (e.g., .gz). Defaults to None.
        """
        self.path = path

        if isinstance(extension, FileType):
            self.extension = extension
        else:
            try:
                self.extension = FileType(extension)
            except Exception as e:
                raise Exception("File Error: This file extension is not supported.")

        self.compression = compression
        self.data_split = data_split



__author__ = "michaellam"

import json
import requests_mock
from pathlib import Path
from aixplain.utils import config
from urllib.parse import urljoin
import pytest

from aixplain.factories.model_factory import ModelFactory

AUTH_FIXED_HEADER = {"Authorization": f"Token {config.TEAM_API_KEY}", "Content-Type": "application/json"}
API_FIXED_HEADER = {"x-api-key": f"{config.TEAM_API_KEY}", "Content-Type": "application/json"}


def test_login():
    url = urljoin(config.BACKEND_URL, f"sdk/ecr/login")
    with requests_mock.Mocker() as mock:
        with open(Path("tests/mock_responses/login_response.json")) as f:
            mock_json = json.load(f)
        mock.post(url, headers=AUTH_FIXED_HEADER, json=mock_json)
        creds = ModelFactory.asset_repo_login(config.TEAM_API_KEY)
    assert creds == mock_json


def test_create_asset_repo():
    url_register = urljoin(config.BACKEND_URL, f"sdk/models/onboard")
    url_function = urljoin(config.BACKEND_URL, f"sdk/functions")
    print(f"URL_Register {url_register}")
    with requests_mock.Mocker() as mock:
        with open(Path("tests/mock_responses/create_asset_repo_response.json")) as f:
            mock_json_register = json.load(f)
        mock.post(url_register, headers=API_FIXED_HEADER, json=mock_json_register, status_code=201)

        with open(Path("tests/mock_responses/list_functions_response.json")) as f:
            mock_json_functions = json.load(f)
        mock.get(url_function, headers=AUTH_FIXED_HEADER, json=mock_json_functions)

        model_id = ModelFactory.create_asset_repo(
            "mock_name", "mock_description", "Text Generation", "en", "text", "text", api_key=config.TEAM_API_KEY
        )
        # print(f"Model ID {model_id}")
    assert model_id == mock_json_register


def test_list_host_machines():
    url = urljoin(config.BACKEND_URL, f"sdk/hosting-machines")
    with requests_mock.Mocker() as mock:
        with open(Path("tests/mock_responses/list_host_machines_response.json")) as f:
            mock_json = json.load(f)
        mock.get(url, headers=API_FIXED_HEADER, json=mock_json)
        machines = ModelFactory.list_host_machines(config.TEAM_API_KEY)
    for i in range(len(machines)):
        machine_dict = machines[i]
        mock_json_dict = mock_json[i]
        for key in machine_dict.keys():
            assert machine_dict[key] == mock_json_dict[key]

def test_get_functions():
    url = urljoin(config.BACKEND_URL, f"sdk/functions")
    with requests_mock.Mocker() as mock:
        with open(Path("tests/mock_responses/list_functions_response.json")) as f:
            mock_json = json.load(f)
        mock.get(url, headers=AUTH_FIXED_HEADER, json=mock_json)
        functions = ModelFactory.list_functions(config.TEAM_API_KEY)
    assert functions == mock_json

@pytest.mark.skip(reason="Not currently supported.")
def test_list_image_repo_tags():
    model_id = "mock_id"
    url = urljoin(config.BACKEND_URL, f"sdk/models/{model_id}/images")
    with requests_mock.Mocker() as mock:
        with open(Path("tests/mock_responses/list_image_repo_tags_response.json")) as f:
            mock_json = json.load(f)
        mock.get(url, headers=AUTH_FIXED_HEADER, json=mock_json)
        tags = ModelFactory.list_image_repo_tags(model_id, config.TEAM_API_KEY)
    assert tags == mock_json



__author__ = "aiXplain"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Author: Duraikrishna Selvaraju, Thiago Castro Ferreira, Shreyas Sharma and Lucas Pavanelli
Date: December 27th 2022
Description:
    Asset Class
"""
from aixplain.enums.license import License
from aixplain.enums.supplier import Supplier
from aixplain.enums.privacy import Privacy
from typing import Dict, Optional, Text, Union


class Asset:
    def __init__(
        self,
        id: Text,
        name: Text,
        description: Text,
        supplier: Union[Dict, Text, Supplier, int] = "aiXplain",
        version: Text = "1.0",
        license: Optional[License] = None,
        privacy: Privacy = Privacy.PRIVATE,
        cost: Optional[Union[Dict, float]] = None,
    ) -> None:
        """Create an Asset with the necessary information

        Args:
            id (Text): ID of the Asset
            name (Text): Name of the Asset
            description (Text): Description of the Asset
            supplier (Union[Dict, Text, Supplier, int], optional): supplier of the asset. Defaults to "aiXplain".
            version (Optional[Text], optional): asset version. Defaults to "1.0".
            cost (Optional[Union[Dict, float]], optional): asset price. Defaults to None.
        """
        self.id = id
        self.name = name
        self.description = description
        try:
            if isinstance(supplier, Supplier) is True:
                self.supplier = supplier
            elif isinstance(supplier, Dict) is True:
                self.supplier = Supplier(supplier)
            else:
                self.supplier = None
                for supplier_ in Supplier:
                    if supplier.lower() in [supplier_.value["code"].lower(), supplier_.value["name"].lower()]:
                        self.supplier = supplier_
                        break
                if self.supplier is None:
                    self.supplier = supplier
        except Exception:
            self.supplier = str(supplier)
        self.version = version
        self.license = license
        self.privacy = privacy
        self.cost = cost

    def to_dict(self) -> dict:
        """Get the asset info as a Dictionary

        Returns:
            dict: Asset Information
        """
        return self.__dict__



__author__ = "thiagocastroferreira"

"""
Copyright 2022 The aiXplain SDK authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

from dotenv import load_dotenv
from urllib.parse import urljoin
import requests_mock

load_dotenv()
import re
from aixplain.utils import config
from aixplain.modules import Model

import pytest


def test_success_poll():
    with requests_mock.Mocker() as mock:
        poll_url = "https://models.aixplain.com/api/v1/data/a90c2078-edfe-403f-acba-d2d94cf71f42"
        headers = {"x-api-key": config.TEAM_API_KEY, "Content-Type": "application/json"}
        ref_response = {"completed": True, "data": "Concluded with success"}
        mock.get(poll_url, headers=headers, json=ref_response)
        test_model = Model("", "")
        hyp_response = test_model.poll(poll_url=poll_url)
    assert hyp_response["completed"] == ref_response["completed"]
    assert hyp_response["status"] == "SUCCESS"


def test_failed_poll():
    with requests_mock.Mocker() as mock:
        poll_url = "https://models.aixplain.com/api/v1/data/a90c2078-edfe-403f-acba-d2d94cf71f42"
        headers = {"x-api-key": config.TEAM_API_KEY, "Content-Type": "application/json"}
        ref_response = {
            "completed": True,
            "error": "err.supplier_error",
            "supplierError": re.escape(
                '{"error":{"message":"The model `<supplier_model_id>` does not exist","type":"invalid_request_error","param":null,"code":"model_not_found"}}'
            ),
        }
        mock.get(poll_url, headers=headers, json=ref_response)
        test_model = Model("", "")
        hyp_response = test_model.poll(poll_url=poll_url)
    assert hyp_response["completed"] == ref_response["completed"]
    assert hyp_response["error"] == ref_response["error"]
    assert hyp_response["supplierError"] == ref_response["supplierError"]
    assert hyp_response["status"] == "FAILED"


@pytest.mark.parametrize(
    "status_code,error_message",
    [
        (401, "Unauthorized API key: Please verify the spelling of the API key and its current validity."),
        (465, "Subscription-related error: Please ensure that your subscription is active and has not expired."),
        (475, "Billing-related error: Please ensure you have enough credits to run this model. "),
        (485, "Supplier-related error: Please ensure that the selected supplier provides the model you are trying to access."),
        (495, "Validation-related error: Please ensure all required fields are provided and correctly formatted."),
        (501, "Status 501: Unspecified error: An unspecified error occurred while processing your request."),
    ],
)

def test_run_async_errors(status_code, error_message):
    base_url = config.MODELS_RUN_URL
    model_id = "model-id"
    execute_url = urljoin(base_url, f"execute/{model_id}")

    with requests_mock.Mocker() as mock:
        mock.post(execute_url, status_code=status_code)
        test_model = Model(id=model_id, name="Test Model", url=base_url)
        response = test_model.run_async(data="input_data")
    assert response["status"] == "FAILED"
    assert response["error_message"] == error_message

